/*
 *  Copyright (c) 1992, 1993 John E. Davis  (davis@amy.tch.harvard.edu)
 *  All Rights Reserved.
 */

#include <stdio.h>
#include "sysdep.h"

#include <signal.h>
/* sequent support thanks to Kenneth Lorber <keni@oasys.dt.navy.mil> */
#ifndef sequent
# include <stdlib.h>
#endif
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#ifndef sequent
# include <termios.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#define TTY_DESCR 2
#ifdef sequent
struct ttystuff
  {
      struct tchars t;
      struct ltchars lt;
      struct sgttyb s;
  };
struct ttystuff OLDTTY;
#else
struct termios OLDTTY;
#endif

     /* this next works on ultrix for setting termios */
#ifdef TCGETS
#define GET_TERMIOS(fd, x) ioctl(fd, TCGETS, x)
#define SET_TERMIOS(fd, x) ioctl(fd, TCSETS, x)
#else
# ifdef sequent
#  define X(x,m)  &(((struct ttystuff*)(x))->m)
#  define GET_TERMIOS(fd, x)	\
	if(ioctl(fd, TIOCGETC, X(x,t))<0 || \
	ioctl(fd, TIOCGLTC, X(x,lt))<0 || \
	ioctl(fd, TIOCGETP, X(x,s))<0)exit_error("Can't get terminal info")
#  define SET_TERMIOS(fd, x)	\
	if(ioctl(fd, TIOCSETC, X(x,t))<0 || \
	ioctl(fd, TIOCSLTC, X(x,lt))<0 || \
	ioctl(fd, TIOCSETP, X(x,s))<0)exit_error("Can't set terminal info")
# else
#  define GET_TERMIOS(fd, x) tcgetattr(fd, x)
#  define SET_TERMIOS(fd, x) tcsetattr(fd, TCSANOW, x)
# endif
#endif

static int Baud_Rates[20] = 
{
   0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800, 
   9600, 19200, 38400, 0, 0, 0, 0
};


void init_tty()
{
#ifdef sequent
    struct ttystuff newtty;
#else
    struct termios newtty;
#endif

   if (Batch) return;
   send_string_to_term("\033=\033[?1l");   /* application keys/cursor keys */
   GET_TERMIOS(TTY_DESCR, &OLDTTY);
   GET_TERMIOS(TTY_DESCR, &newtty);
#ifdef sequent
   newtty.s.sg_flags &= ~(ECHO);
   newtty.s.sg_flags &= ~(CRMOD);
   newtty.t.t_eofc = 1;
   newtty.t.t_intrc = Abort_Char;	/* ^G */
   newtty.t.t_quitc = 255;
   newtty.lt.t_suspc = 255;   /* to ignore ^Z */
   newtty.lt.t_lnextc = 255;
   newtty.s.sg_flags |= CBREAK;		/* do I want cbreak or raw????? */
#else
   
   
   if (!Output_Rate)
     {
/* Note:  if this generates an compiler error, simply remove 
   the statement */
	Output_Rate = cfgetospeed (&newtty);
	
	
	Output_Rate = (Output_Rate > 0) && (Output_Rate < 19) ?
	              Baud_Rates[Output_Rate] / 10 : 4096;
     }
   
   newtty.c_iflag &= ~(ECHO);
   newtty.c_iflag &= ~(INLCR);
   newtty.c_iflag &= ~(ICRNL);
   newtty.c_oflag &= ~ONLCR;	       /* do not map newline to cr/newline on out */
   newtty.c_cc[VMIN] = 1;
   newtty.c_cc[VTIME] = 0;
   newtty.c_cc[VEOF] = 1;
   newtty.c_lflag = ISIG | NOFLSH;
   newtty.c_cc[VINTR] = Abort_Char;   /* ^G */
   newtty.c_cc[VQUIT] = 255;
   newtty.c_cc[VSUSP] = 255;   /* to ignore ^Z */
#endif /*sequent*/
   SET_TERMIOS(TTY_DESCR, &newtty);
}

void reset_tty()
{
   if (Batch) return;
   send_string_to_term("\033>");  /* Enable the Numeric Keypad */
   SET_TERMIOS(TTY_DESCR, &OLDTTY);
}

unsigned char sys_getkey()
{
   int n = 60;
   unsigned char c;
   if (KeyBoard_Quit) return((int) Abort_Char);
   while (!KeyBoard_Quit && !input_pending(&n));
   /* sleep for 60 second and try again */
   if (KeyBoard_Quit) return(Abort_Char);
   if ((read(TTY_DESCR, &c, 1) < 0) && !KeyBoard_Quit)
     {
	fprintf(stderr,"getkey():  read failed.\n");
	reset_tty();
	exit(1);
     }
   /* only way for keyboard quit to be non zero is if ^G recived and sigint processed */
   if (KeyBoard_Quit) c = Abort_Char;
   KeyBoard_Quit = 0;

   return(c);
}

int input_pending(int *secs)
{
   struct timeval wait;
   int readfd, writefd, exceptfd, ret;
   long usecs = 0;
   int fd = 2;
   /* extern int select(int, int *, int *, int *, struct timeval *); */

   if (INPUT_BUFFER_LEN || Batch) return(INPUT_BUFFER_LEN);

   /* if (*secs == 0) usecs = 1000; */  /* 1/1000th of a sec */
   wait.tv_sec = *secs;
   wait.tv_usec = usecs;
   
   readfd = 1 << fd;
   writefd = exceptfd = 0;
   
   ret = select(fd + 1, &readfd, &writefd, &exceptfd, &wait);
   return(ret);
}

/*  This is to get the size of the terminal  */
void get_term_dimensions(int *cols, int *rows)
{
#ifdef TIOCGWINSZ
   struct winsize wind_struct;

   ioctl(2,TIOCGWINSZ,&wind_struct);
   *cols = (int) wind_struct.ws_col;
   *rows = (int) wind_struct.ws_row;
   if (*rows <= 0) *rows = 24;
   if (*cols <= 0) *cols = 80;
#else
   *rows = 24;
   *cols = 80;
#endif
}

/* returns 0 on failure, 1 on sucess */
int sys_delete_file(char *filename)
{
    return(1 + unlink(filename));
}

int sys_rename(char *from, char *to)
{
    return(rename(from,to));
}

void sys_suspend(void)
{
#ifdef _HPUX_SOURCE
    kill(getpgrp(),SIGTSTP);
#else
#ifdef _POSIX_SOURCE
   kill(getpgrp(),SIGTSTP);
#else 
#ifdef sgi
   kill(getpgrp(),SIGTSTP);
#else
   kill(getpgrp(),SIGTSTP);
#endif
#endif
#endif
   /* According to man pages, sys5 takes no argument for getpgrp but BSD does */
}

/* returns 0 if file does not exist, 1 if it is not a dir, 2 if it is */
int sys_chmod(char *file, int what, int *mode)
{
   struct stat buf;
   int m;

   if (what)
     {
	chmod(file, *mode);
	return(0);
     }

   if (stat(file, &buf) < 0) switch (errno)
     {
	case EACCES: return(-1); /* es = "Access denied."; break; */
	case ENOENT: return(0);  /* ms = "File does not exist."; */
	case ENOTDIR: return(-2); /* es = "Invalid Path."; */
	default: return(-3); /* "stat: unknown error."; break;*/
     }

   m = buf.st_mode;

/* AIX requires this */
#ifdef _S_IFDIR
#ifndef S_IFDIR
#define S_IFDIR _S_IFDIR
#endif
#endif

   *mode = m & 0777;

   if (m & S_IFDIR) return (2);
   return(1);
}

unsigned long sys_file_mod_time(char *file)
{
   struct stat buf;

   if (stat(file, &buf) < 0) return(0);
   return((unsigned long) buf.st_mtime);
}

